PROGRAM CONRED41;
{$F+}
{***********************************************************
"PUENTE DE TIRISTORES TRIFASICO CON INDUCTANCIA SERIE"
Autores: JRomero - Arodri
Fecha: 16-02-98

		 La simulacin corresponde a un PUENTE DE TIRISTORES TRIFASICO
		 CON INDUCTANCIA SERIE

		 Y se observarn los siguientes parmetros:

		 * tensiones de fase de entrada, tension e intensidad de salida.
		 * tensiones de fases junto a la tension de conmutacion y del nodo UA
		 * corriente de los TIRISTORES y corriente de fase R
**********************************************************}

USES
wincrt,xmatdefs,winGraph, Dibus,FilUSR, UCntrl, Simul4, traxp, variable,
plotmems, memorias, lib_basi, libep02, libep, libdig01;

CONST w=PI*100;           {frecuencia de red=50 hz}
		periodo=20e-3;      {periodo=2*PI/w}
		Npuntos=1024;       {nmero de puntos de muestreo}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
var
	L_conmu, alpha:NReal; {.Para controlar variables.}
	cada_10,tension, u, valor_medio_Ud,Intensidad, grados:NReal;  {variables para visualizar en pantalla}
	mem1:Tmemoria;
	enY1, enY2, enY3, enY4, enY5, enX, grad:Integer;

{----------------------------------------------------------------------}

FUNCTION disparo(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
		disparo:=(vref<=sin(w*t-2*PI/3-(k-1)*PI/3)) and (cos(w*t-2*PI/3-(k-1)*PI/3)>=0);

END;

{----------------------------------------------------------------------}
{*********************************************************
A continuacin los procedimientos Cirt1, Cirt2 y Funcsr de
definicin del circuito.
*********************************************************}

PROCEDURE CIRT1;
BEGIN
	  vg(1,6,7, tension);            {R}
	  vg(2,6,8, tension);            {S}
	  vg(3,6,9, tension);     		  {T}

	  L(7,16, L_CONMU);   	{L lnea}
	  L(8,2, L_CONMU);		{L lnea}
	  L(9,3, L_CONMU);		{L lnea}

	  r(7,16,10000);				{r para bobina del primario}
	  r(8,2,10000);				{r para bobina del primario}
	  r(9,3,10000);				{r para bobina del primario}

	  r(16,1, 0.01);            {r para medir iR}

	  r(4,10, 0.01);				{r para medir i de tiristores}
	  r(4,11, 0.01);				{r para medir i de tiristores}
	  r(4,12, 0.01);				{r para medir i de tiristores}

	  r(0,13, 0.01);				{r para medir i de tiristores}
	  r(0,14, 0.01);				{r para medir i de tiristores}
	  r(0,15, 0.01);				{r para medir i de tiristores}


	  ig(0,4,5, 1);             {CARGA}
	  R(5,0, 0.01);
END;

{---------------------------------------------------------------------}

PROCEDURE CIRT2;
BEGIN
	  th(1,10,disparo(1));     {T1}
	  th(15,3,disparo(2));     {T2}
	  th(2,11,disparo(3));     {T3}
	  th(13,1,disparo(4));     {T4}
	  th(3,12,disparo(5));     {T5}
	  th(14,2,disparo(6));     {T6}
END;

{---------------------------------------------------------------------}

PROCEDURE FUNCSR(t:NReal);
BEGIN
rf[1]:=sin(w*t);        	{R}
rf[2]:=sin(w*t -2*PI/3);	{S}
rf[3]:=sin(w*t -4*PI/3);	{T}
END;

{---------------------------------------------------------------------}

{********************************************************
A continuacin los procedimientos de inicio, fin  y Pesca
necesarios para Simeep
*********************************************************}

PROCEDURE INICIO;
BEGIN
nn:=16;			{nmero de nodos del circuito sin incluir 0 (Tierra)}
nindep:=4;		{nmero de fuentes independientes}
tinicial:=0e-3;{tiempo incial de cada simulacin}
tfinal:=20e-3; {tiempo final de cada simulacin}

{Valores iniciales de variables de estado.
 Estas se definen en el orden en que aparecen las inductancias
 y Capacitancias en Cirt1}

x0[1]:=0.027692448661;		{corresponde a Lconm R}
x0[2]:=-1.1426359745;		{corresponde a Lconm S}
x0[3]:=1.1149435258;		{corresponde a Lconm T}


{valores de precisin de Simeep}
dtmax0:=10e-6;
dtmin:=dtmax0/100;
PasoDivisor:=6;
TmaxTmin:=1e3;

{indicamos a TRAXP los tiempos que debe graficar}
traxp.tinicial:=tinicial;
traxp.tfinal:=tfinal;

{En esta seccin, definimos el nmero de pantallas, y los colores,
 en nmero de grillas inciales, y si tenemos alguna superpocicin
 de seales en una misma ventana}

gridx:=12;	{las divisiones en X son 12}
gridy:=8;	{las diviciones en Y son 8}
subplot(1,3); {matriz de pantallas de 1 por 1}

{definiciones de visualizacin para la pantalla 0,est asociada al canal 0}
definaY(0,-600, 150);
titulo('(Azul):Ud.  vRS         vRT         vST         vSR          vTR          vTS');
xlabel('30/DIV');
ylabel('150 V/DIV');
linea0;
grid;
superponga(3,0);
superponga(4,0);
superponga(5,0);
superponga(6,0);
superponga(7,0);
superponga(8,0);
superponga(17,0);
{definimos otros colores para algunos de los canales}
definacolor(0,azul);
definacolor(3,negro);
definacolor(4,violeta);
definacolor(5,12);
definacolor(6,6);
definacolor(7,14);
definacolor(8,7);
definacolor(17,12);

{.....................................................................}
{definiciones de visualizacin para la pantalla 1,est asociada al canal 1}
definaY(1,-360, 90);
titulo('Tension en nodo (Ua) durante la conmutacion de 1 a 3.  Tension de conmutacion ');
xlabel('30/DIV');
ylabel('90 V/DIV.    R-S-T');
linea0;
grid;
superponga(9,1);
superponga(10,1);
superponga(18,1);
superponga(19,1);
 definacolor(1,rojo);
 definacolor(9,negro);
 definacolor(10,azul);
definacolor(18,11);
definacolor(19,14);

{.....................................................................}
{definiciones de visualizacin para la pantalla 2,est asociada al canal 2}
definaY(2,-350, 50);
titulo('(Amarillo):intensidades de tiristores.    (Rojo):intensidad de la fase R');
xlabel('30/DIV');
ylabel('  6  5 4 3  2 1 iR');
linea0;
grid;
{definimos otros colores para algunos de los canales}
superponga(11,2);
superponga(12,2);
superponga(13,2);
superponga(14,2);
superponga(15,2);
superponga(16,2);
 definacolor(2, 14);
 definacolor(11,14);
 definacolor(12,14);
 definacolor(13,14);
 definacolor(14,14);
 definacolor(15,14);
 definacolor(16,ROJO);
END;

{---------------------------------------------------------------------}

PROCEDURE PESCA;
var   k:integer;
		coseno,coseno_absoluto:NReal;
BEGIN
grados:=18000*t; {....Para ver en el display el tiempo y los grados.}
if grados>=360 then grados:=grados-360*Int(t/0.02);

Intensidad:=v[5]/0.01;{Calculo la intensidad de salida instantanea}

mem1.tic(t,v[4]);

cada_10:=cada_10+0.1;
if cada_10>=3 then cada_10:=1;

			  {Calculo el tiempo de conmutacion Uk}
coseno:=cos(alpha*PI/180)-w*L_conmu*2*intensidad/ (SQRT(3)*tension);
if (coseno>-0.000001) AND (coseno<0.000001) then coseno:=0.000001;
coseno_absoluto:=ABS(coseno);
U:=ArcTan(SQRT(ABS(1-SQR(coseno_absoluto)))/coseno_absoluto);
if coseno<0 then U:=PI-U;
u:=u*180/pi;
U:=U-alpha;




{...Pinto anotaciones sobre las graficas...}

enX:=PUNTOX+24; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}

{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{alpha}ANOTAR_EVENTO(1,150,150+alpha,grados, enx,70,38,75,eny1,75, CHR(224));

{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Uu}ANOTAR_EVENTO(2,150+alpha,150+alpha+U,grados, enx,100,eny1,103,eny1,103,'U');


{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{alpha}ANOTAR_EVENTO(3,150,150+alpha,grados, enx,260,218,265,eny1+180,265, CHR(224));

{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Uu}ANOTAR_EVENTO(4,150+alpha,150+alpha+U,grados, enx,280,enY5+158,283, 283,enY5+158,'U');

{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{180}ANOTAR_EVENTO(5,150,150+180,grados, enx,295,235,295,298,235,'   180');


{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Ui}ANOTAR_EVENTO(6,150+alpha,150+alpha+U,grados, enx,420,400,425,394,425,'U');

{ANOTAR_TEXTO(N,GRAD1,GRADOS, enx,Y,color, ETIQUETA);}
ANOTAR_TEXTO(1,90,GRADOS,ENX-5, 192,ROJO, 'UR');
ANOTAR_TEXTO(2,210,GRADOS,ENX-5, 192,NEGRO, 'US');
ANOTAR_TEXTO(3,330,GRADOS,ENX-5, 192,AZUL, 'UT');
ANOTAR_TEXTO(4,240,GRADOS,ENX-5, 201,14, 'uk');
ANOTAR_TEXTO(5,150+alpha+5,GRADOS,ENX-5, enY5+142,10, 'ua');


{indicar el tiempo de simulacin a el graficador}
traxp.t:=t;

{relacin de seales a graficar con los canales deseados}
{...................grafica 0 ..........................}

trazo(0,v[4]);                   { TENSION EN LA CARGA R-L }
trazo(0,0);
eny1:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
RELLENAR_EVENTO(150+alpha-1, 150+alpha+U, grados,enx, enY1, enY2,    13);

{trazo(17,v[4]);            { Tension en la carga R-L }

trazo(3,v[7]-v[8]);              { TENSION de fase R-S }


trazo(4,v[7]-v[9]);              { TENSION de fase R-T }

trazo(5,v[8]-v[9]);              { TENSION de fase S-T }
eny2:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
trazo(6,v[8]-v[7]);              { TENSION de fase S-R }
trazo(7,v[9]-v[7]);              { TENSION de fase T-R }
trazo(8,v[9]-v[8]);              { TENSION de fase T-S }

{...................grafica 1 ..........................}

if (grados>30+alpha+u) and (grados<30+alpha+120) then SetLineStyle(0,0,3);
trazo(1, v[7]- v[6]);            { TENSION linea R }
eny3:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
SetLineStyle(0,0,0);


if (grados>150+alpha+u) and (grados<150+alpha+120) then SetLineStyle(0,0,3);
trazo(9,v[8] -v[6]);             { TENSION linea S }
eny4:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
SetLineStyle(0,0,0);


if (grados>90+alpha+u) and (grados<90+alpha+120) then SetLineStyle(0,0,3);
trazo(10,v[9] -v[6]);   { TENSION linea T }
SetLineStyle(0,0,0);

 if (grados>150) and (grados<360) then {tension uk  rojo y blanco}
 begin
 if odd(trunc(cada_10)) then
							  definacolor(19,13)
							  else
							  definacolor(19,14);
 trazo(19,((v[8]-v[6])-(v[7]-v[6]))*0.3 );
 end;



 if (grados>150) and (grados<150+alpha+u) then{tension nodo Ua a trazos verde}
 begin
	if (Odd(Trunc(cada_10))) or (grados>150+alpha) and (grados<150+alpha+u)
										 then
										 begin
										  if (grados>150+alpha) and (grados<150+alpha+u)
										  then SetLineStyle(0,0,3);
										 definacolor(18,10);
										 end
										  else
										 definacolor(18,3);
 trazo(18,(v[8]-v[6] + v[7]-v[6])/2 );{TENSION en nodo Ua}
 eny5:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
 SetLineStyle(0,0,0);
 end;



{...................grafica 2 ..........................}
trazo(2,  (v[10]-v[4])   *2200-50);         {  i t1 }
trazo(2,0-50);
trazo(11,  (v[0] -v[15]) *2200-100);        {  i t2 }
trazo(11,0-100);
trazo(12, (v[11]-v[4])   *2200-150);        {  i t3 }
trazo(12,0-150);
trazo(13, (v[0] -v[13])  *2200-200);        {  i t4 }
trazo(13,0-200);
trazo(14, (v[12]-v[4])   *2200-250);        {  i t5 }
trazo(14,0-250);
trazo(15, (v[0] -v[14])  *2200-300);        {  i t6 }
trazo(15,0-300);

trazo(16, (v[16] -v[1])  *2200);           { i linea R }
trazo(16,0);




	{borrado de la pantalla despus de cada simulacin}
	if t >= tfinal then
	begin
		for k:= 0 to 19 do
		begin
		borrecanal(k,0);
		desligue(k);
		borde;
		grid;
		linea0;
		mem1.fft;
		end;
		valor_medio_Ud:=mem1.valormedio;
		mem1.retrocesobarrido(tfinal-tinicial);

		t:= t-tfinal+tinicial;
	end;
END;

{---------------------------------------------------------------------}

PROCEDURE FIN;
BEGIN
	halt;
END;

{---------------------------------------------------------------------}

PROCEDURE INICIALIZACION_PARAMETROS;
BEGIN
{valores iniciales de los controles}
tension:=311;
cada_10:=1;
END;

{---------------------------------------------------------------------}


{----------------------------------------------------------------------}
{******************************PRINCIPAL*******************************}
{----------------------------------------------------------------------}
BEGIN
INICIALIZACION_PARAMETROS;
{Indicamos a Simeep donde estn las funciones que preparamos}
FilUSR.Cirt1:=Cirt1;
FilUSR.Cirt2:=Cirt2;
FilUSR.FuncsR:=FuncsR;
FilUSR.Inicio:=Inicio;
FilUSR.Pesca:=Pesca;
FilUSR.Fin:=Fin;

{Pantalla de controles}
UCntrl.init;
mem1:= TMemoria.Create_init(20e-3/1024,1024,0,0);

{------------------PARA VISUALIZAR TEXTO EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlVerTexto,init
(    0,   'RECTIFICADOR TRIFASICO',   ' PUENTE DE TIRISTORES',
			  'CON INDUCTANCIA SERIE' )));

UCntrl.insert(new(PCtrlVerTexto,init
(    1,   '    ',
			 '        CARGA',
			 '    FUENTE I=Cte.' )));
UCntrl.insert(new(PCtrlVerTexto,init
(    2,   '    ',
			 '    VISUALIZACION    ',
			 '    DE  VARIABLES ')));



UCntrl.insert(new(PCtrlVerTexto,init
(    6,   '    ',
			 '     ',
			 'INDUC.SERIE Y DISPARO ')));


{---------------PARA CONTROLAR VARIABLES EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    7, ' L_CONMU',  'mH'    ,0,       1,  100,  30, 10, L_CONMU)));


UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    8, '  Alpha',  ''    ,0,       0,   90,    30,  2,   alpha)));


{-----------------PARA VER VARIABLES EN LA SIMULACION------------------}

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon,  nombre1,  uni1, deci1,  var1 , nombre2,  uni2, deci2, var2}
(    3,    'Tiempo',   'Sg',   3  ,  t ,  'Grados',  '',    0 ,Grados)));


UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    5, 'Ud_inst.',  'V',     1  , v[4] , 'Ud_med.', 'A',   3 , valor_medio_Ud)));

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    4, 'u',  '',   1  , U , 'Id_carga', 'A',   3 ,cada_10)));


{----------------------------------------------------------------------}

{Inicio de Simulaciones}
PASEO;
END.
