PROGRAM CONRED61;
{$F+}
{***********************************************************
"CONVERTIDOR 12 PULSOS DOS VIAS CON INDUCTANCIA SERIE Y CARGA a I=Cte"
Autores: JRomero - Arodri
Fecha: 16-02-98

		 La simulacin corresponde a un CONVERTIDOR 12 PULSOS DOS VIAS
		 CON INDUCTANCIA DE LINEA Y COORTOCICUITO DEL SECUNDARIO DEL
		 TRANSFORMADOR Y CARGA a I=Cte.

		 y se observarn los siguientes parmetros:

		 * intensidad absorvida desde la red.
		 * tension de fase de red.
		 * Contenido armonico en dicha intensidad.
**********************************************************}

USES
wincrt,xmatdefs, DIBUS,FilUSR, UCntrl, Simul4, traxp, variable,
PlotMems, memorias, lib_basi, libep02, libep, libdig01;

CONST w=PI*100;           {frecuencia de red=50 hz}
		periodo=20e-3;      {periodo=2*PI/w}
		Npuntos=1024;       {nmero de puntos de muestreo}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}

var
	mem1:Tmemoria;
	mem2:Tmemoria;
	L_conmu:NReal; {.Para controlar variables.}
	U, IR_RED, Intensidad, grados:NReal;  {variableS para visualizar en pantalla}
	valor_medio_Ud:NReal;

{variables de mando que almacenan los valores introducidos por el usuario}
	Alpha:NReal;
	inductor:NReal;
{----------------------------------------------------------------------}

FUNCTION disparo1(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
		disparo1:=(vref<=sin(w*t-2*PI/3 -PI/6 -(k-1)*PI/3)) and
						 (cos(w*t-2*PI/3 -PI/6 -(k-1)*PI/3)>=0);
END;

{----------------------------------------------------------------------}
FUNCTION disparo2(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
		disparo2:= (vref<=sin (w*t-2*PI/3 -(k-1)*PI/3)) and
						(cos(w*t-2*PI/3  -(k-1)*PI/3)>=0);
END;

{----------------------------------------------------------------------}

{*********************************************************
A continuacin los procedimientos Cirt1, Cirt2 y Funcsr de
definicin del circuito.
*********************************************************}

PROCEDURE CIRT1;
BEGIN
	  vg(1, 14,13, 311*SQRT(3));   {R1}
	  vg(2, 12,14, 311*SQRT(3));   {S1}
	  vg(3, 13,15, 311*SQRT(3));   {T1}

	  vg(4, 10,17, 311);   {R2}
	  vg(5, 10,18, 311);   {S2}
	  vg(6, 10,19, 311);   {T2}

	  L(13,16, L_conmu);  {L de linea del 1}
	  L(14,21, L_conmu);
	  L(15,3, L_conmu);
	  R(13,16, 1000);
	  R(14,21,  1000);
	  R(15,3,  1000);

	  R(16,1,  0.01);  {para medir iU del 1}
	  R(21,2,  0.01);  {para medir iV del 1}

	  L(17,20, L_conmu);  {L de linea del 2}
	  L(18,7, L_conmu);
	  L(19,8, L_conmu);
	  R(17,20, 1000);
	  R(18,7,  1000);
	  R(19,8,  1000);

	  R(20,6,  0.01);  {para medir iX del 2}

	  R(4,11, 10);           { R PARALELO CON L}
	  IG(0,11,0, 1);

	  R(9,5,  1);           { R DE UNION}

END;

{---------------------------------------------------------------------}

PROCEDURE CIRT2;
BEGIN
	  th(1,4,disparo1(1));     {T1}
	  th(5,3,disparo1(2));     {T2}
	  th(2,4,disparo1(3));     {T3}
	  th(5,1,disparo1(4));     {T4}
	  th(3,4,disparo1(5));     {T5}
	  th(5,2,disparo1(6));     {T6}

	  th(6,9,disparo2(1));     {T1}
	  th(0,8,disparo2(2));     {T2}
	  th(7,9,disparo2(3));     {T3}
	  th(0,6,disparo2(4));     {T4}
	  th(8,9,disparo2(5));     {T5}
	  th(0,7,disparo2(6));     {T6}
END;

{---------------------------------------------------------------------}

PROCEDURE FUNCSR(t:NReal);
BEGIN
rf[1]:=sin(w*t);           {R1}
rf[2]:=sin(w*t-2*PI/3 );   {S1}
rf[3]:=sin(w*t-4*PI/3 );   {T1}

rf[4]:=sin(w*t );       	{R2}
rf[5]:=sin(w*t-2*PI/3 );	{S2}
rf[6]:=sin(w*t-4*PI/3 );	{T2}
END;

{---------------------------------------------------------------------}

{********************************************************
A continuacin los procedimientos de inicio, fin  y Pesca
necesarios para Simeep
*********************************************************}

PROCEDURE INICIO;
BEGIN
nn:=21;			{nmero de nodos del circuito sin incluir 0 (Tierra)}
nindep:=6;		{nmero de fuentes independientes}
tinicial:=0;   {tiempo incial de cada simulacin}
tfinal:=20e-3; {tiempo final de cada simulacin}

{Valores iniciales de variables de estado.
 Estas se definen en el orden en que aparecen las inductancias
 y Capacitancias en Cirt1}

x0[1]:=-1.093063195;
x0[2]:=-0.046835731277;
x0[3]:=1.1398989263;
x0[4]:=0.00021276838149;
x0[5]:=-1.1347433887;
x0[6]:=1.1345306203;

{valores de precisin de Simeep}
dtmax0:=20e-6;
dtmin:=dtmax0/100;
PasoDivisor:=6;
TmaxTmin:=1e3;

{indicamos a TRAXP los tiempos que debe graficar}
traxp.tinicial:=tinicial;
traxp.tfinal:=tfinal;

{En esta seccin, definimos el nmero de pantallas, y los colores,
 en nmero de grillas inciales, y si tenemos alguna superpocicin
 de seales en una misma ventana}

gridx:=12;	{las divisiones en X son 12}
gridy:=12;	{las diviciones en Y son 8}
subplot(1,2); {matriz de pantallas de 1 por 1}

{definiciones de visualizacin para la pantalla 0 y est asociada al canal 0}
definaY(0,-360,60);
titulo(' (Amarillo): Intensidad absorvida de la red i_R    (Rojo): Tensin fase R');
xlabel(' ENTRADA');
ylabel('60 V/DIV.     0.4 A/DIV');
linea0;
grid;
superponga(2,0);
 definacolor(0,14);
 definacolor(2,rojo);

{definiciones de visualizacin para la pantalla 1 y est asociada al canal 1}
definaY(1,-500,50);
titulo(' Armnicos de la intensidad absorvida por el convertidor de la red');
xlabel('0 1      5        10        15       20        25       30        35      40');
ylabel('');
END;

{---------------------------------------------------------------------}

PROCEDURE PESCA;
var   k, enX:integer;
		coseno:NREAL;
BEGIN

grados:=18000*t; {....Para ver en el display el tiempo y los grados.}
if grados>=360 then grados:=grados-360*Int(t/0.02);

iR_RED:=(v[20]-v[6])/(2*0.01)+(V[16]-V[1]-V[21]+V[2])/(0.01*2*sqrt(3));
mem1.tic(t,iR_RED);
mem2.tic(t,v[4]);

intensidad:= (v[4]-v[11])/10;


			  {Calculo el tiempo de conmutacion Uk y de fi}
coseno:=cos(alpha*PI/180)-w*L_conmu*2*intensidad/ (SQRT(3)*311);
if (coseno>-0.00001) AND (coseno<0.000001) then coseno:=0.000001;
coseno:=ABS(coseno);
U:=ArcTan(SQRT(ABS(1-SQR(coseno)))/coseno);
if alpha>90 then U:=PI-U;
u:=u*180/pi;
U:=ABS(U-alpha);


{...Pinto anotaciones sobre las graficas...}
enX:=PUNTOX+25; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}

{ANOTAR_EVENTO(GRAD1,GRAD2,GRADOS, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Alpha}ANOTAR_EVENTO(1,180, 180+alpha, grados,enx,60,57,120,57,120,CHR(224));


{indicar el tiempo de simulacin a el graficador}
traxp.t:=t;
{relacin de seales a graficar con los canales deseados}

{...........grafica 0..........}
trazo(0, iR_red*150);              { i red R }
trazo(0,0);
trazo(2,  (v[17]-V[10]));             { v red R }

	{borrado de la pantalla despus de cada simulacin}
	if t >= tfinal then
	begin
			for k:= 0 to 2 do
		begin
		borrecanal(k,0);
		desligue(k);
		borde;
		if k<>1 then
		begin
			grid;
			linea0;
		end;
		end;
		mem1.fft;
		mem2.fft;
		{...........grafica 5..........}
		{graficar: Armonicos de corriente de la fase R de alimentacion}
		gridx:=41;
		PlotMem_EspectroUnBAR(1,mem1,0,40,  12,1);
		gridx:=12;	{las divisiones en X son 12}
		gridy:=12;	{las diviciones en Y son 8}
		mem1.retrocesobarrido(tfinal-tinicial);
		valor_medio_Ud:=mem2.valormedio;
		mem2.retrocesobarrido(tfinal-tinicial);

		t:= t-tfinal+tinicial;
	end;
END;

{---------------------------------------------------------------------}

PROCEDURE FIN;
BEGIN
HALT;
END;

{---------------------------------------------------------------------}

PROCEDURE INICIALIZACION_PARAMETROS;
BEGIN
valor_medio_ud:=983;
END;

{---------------------------------------------------------------------}


{******************************PRINCIPAL*******************************}
BEGIN
INICIALIZACION_PARAMETROS;

{Indicamos a Simeep donde estn las funciones que preparamos}
FilUSR.Cirt1:=Cirt1;
FilUSR.Cirt2:=Cirt2;
FilUSR.FuncsR:=FuncsR;
FilUSR.Inicio:=Inicio;
FilUSR.Pesca:=Pesca;
FilUSR.Fin:=Fin;

{Pantalla de controles}
UCntrl.init;

mem1:= TMemoria.Create_init(20e-3/1024,1024,0,0);
mem2:= TMemoria.Create_init(20e-3/1024,1024,0,0);
{------------------PARA VISUALIZAR TEXTO EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlVerTexto,init
(    0,   '    CONVERTIDOR ',
			 '    EN CONEXION',
			 '12 PULSOS  DOS VIAS ')));

UCntrl.insert(new(PCtrlVerTexto,init
(    1,   '     ',
			 '        CARGA',
			 '   FUENTE DE I=Cte. ')));

UCntrl.insert(new(PCtrlVerTexto,init
(    6,   '    ',
			 '     ',
			 'INDUC.SERIE Y DISPARO ')));


{---------------PARA CONTROLAR VARIABLES EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    7, ' L_CONMU',  'mH'    ,0,       1,  100,  10, 10, L_CONMU)));


UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    8, ' Disparo',  ''    ,0,       1,   90,    15,  2,   alpha)));

{-----------------PARA VER VARIABLES EN LA SIMULACION------------------}

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon,  nombre1,  uni1, deci1,  var1 , nombre2,  uni2, deci2, var2}
(    2,    'Tiempo',   'Sg',   3  ,  t ,  'Grados',  '',    0 ,Grados)));


UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    3, 'u',  '',      1  ,  U , 'i_d' ,  'A',   1 , intensidad)));



UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    4, 'v_d med.',  'V',     1  , valor_medio_ud , 'i_R inst.', 'A',  1, iR_red)));


PASEO;
mem1.Free;
END.
