PROGRAM CONRED65;
{$F+}
{***********************************************************
"CONVERTIDOR 12 PULSOS DOS VIAS CON INDUCTANCIA SERIE Y CARGA I=Cte"
Autores: JRomero - Arodri
Fecha: 16-02-98

		 La simulacin corresponde a un CONVERTIDOR 12 PULSOS DOS VIAS
		 CON INDUCTANCIA DE LINEA Y COORTOCICUITO DEL SECUNDARIO DEL
		 TRANSFORMADOR Y CARGA a I=Cte

		 y se observarn los siguientes parmetros:

		 * tensiones de alimentacion de fase y entre fases.
		 * intensidades por fase de cada convertidor
		 * intensidad de fase total junto a la intensidad de fase total.
		 * tensin de salida de cada convertidor.
		 * tension de salida total.
		 * corriente de carga.
		 * estado de los tiristores.
		 * Contenido armonico de la intensidad de fase.

{***********************************************************}

USES
wincrt,xmatdefs, winGraph,DIBUS,FilUSR, UCntrl, Simul4, traxp, variable,
PlotMems, memorias, lib_basi, libep02, libep, libdig01;

CONST w=PI*100;           {frecuencia de red=50 hz}
		periodo=20e-3;      {periodo=2*PI/w}
		Npuntos=1024;       {nmero de puntos de muestreo}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
var
	mem1:Tmemoria;         {para calculo de armonicos}
	mem2:Tmemoria;         {para calculo del valor medio de la Ud}

	L_conmu:NReal;         {.Para controlar variables.}
	tension,u, IR_RED, Intensidad, grados:NReal;  {variables para visualizar en pantalla}
	valor_medio_Ud:NReal;

{variables de mando que almacenan los valores introducidos por el usuario}
	 Alpha:NReal;
	 inductor:NReal;
	 conmutacion:Boolean;

{----------------------------------------------------------------------}

FUNCTION disparo1(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
			disparo1:=(vref<=sin(w*t-2*PI/3 -PI/6 -(k-1)*PI/3)) and
						 (cos(w*t-2*PI/3 -PI/6 -(k-1)*PI/3)>=0);
END;

{----------------------------------------------------------------------}
FUNCTION disparo2(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
		disparo2:= (vref<=sin (w*t-2*PI/3 -(k-1)*PI/3))	and
					  (cos(w*t-2*PI/3  -(k-1)*PI/3)>=0);

END;

{----------------------------------------------------------------------}

{*********************************************************
A continuacin los procedimientos Cirt1, Cirt2 y Funcsr de
definicin del circuito.
*********************************************************}

PROCEDURE CIRT1;
BEGIN
	  vg(1, 14,13, tension*SQRT(3));   {R1}
	  vg(2, 12,14, tension*SQRT(3));   {S1}
	  vg(3, 13,15, tension*SQRT(3));   {T1}

	  vg(4, 10,17, tension);   {R2}
	  vg(5, 10,18, tension);   {S2}
	  vg(6, 10,19, tension);   {T2}

	  if conmutacion=true then
	  begin
	  L(13,16, L_conmu);  {L de linea del 1}
	  L(14,21, L_conmu);
	  L(15,3,  L_conmu);
	  R(13,16, 1000);
	  R(14,21, 1000);
	  R(15,3,  1000);

	  L(17,20, L_conmu);  {L de linea del 2}
	  L(18,7,  L_conmu);
	  L(19,8,  L_conmu);
	  R(17,20, 1000);
	  R(18,7,  1000);
	  R(19,8,  1000);
	  end
	  else
	  begin
	  R(13,16, L_conmu); {las inductancias son ahora resistencias}
	  R(14,21, L_conmu);
	  R(15,3,  L_conmu);
	  R(17,20, L_conmu);
	  R(18,7,  L_conmu);
	  R(19,8,  L_conmu);
	  end;

	  R(16,1,  0.01);  {para medir iU del 1}
	  R(21,2,  0.01);  {para medir iV del 1}
	  R(20,6,  0.01);  {para medir iX del 2}

	  r(4,11, 0.1);           {R de carga}
	  Ig(0,11,0,1);        {en la carga}

	  R(9,5,  1);           {R DE UNION entre convertidores}
END;

{---------------------------------------------------------------------}

PROCEDURE CIRT2;
BEGIN
	  th(1,4,disparo1(1));     {T1}   {convertidor A}
	  th(5,3,disparo1(2));     {T2}
	  th(2,4,disparo1(3));     {T3}
	  th(5,1,disparo1(4));     {T4}
	  th(3,4,disparo1(5));     {T5}
	  th(5,2,disparo1(6));     {T6}

	  th(6,9,disparo2(1));     {T1}   {convertidor B}
	  th(0,8,disparo2(2));     {T2}
	  th(7,9,disparo2(3));     {T3}
	  th(0,6,disparo2(4));     {T4}
	  th(8,9,disparo2(5));     {T5}
	  th(0,7,disparo2(6));     {T6}
END;

{---------------------------------------------------------------------}

PROCEDURE FUNCSR(t:NReal);
BEGIN
rf[1]:=sin(w*t);        	{R1}
rf[2]:=sin(w*t-2*PI/3 ); 	{S1}
rf[3]:=sin(w*t-4*PI/3 );	{T1}

rf[4]:=sin(w*t ); 	      {R2}
rf[5]:=sin(w*t-2*PI/3 );	{S2}
rf[6]:=sin(w*t-4*PI/3 );	{T2}
END;

{---------------------------------------------------------------------}

{********************************************************
A continuacin los procedimientos de inicio, fin  y Pesca
necesarios para Simeep
*********************************************************}

PROCEDURE INICIO;
BEGIN
nn:=21;			{nmero de nodos del circuito sin incluir 0 (Tierra)}
nindep:=6;		{nmero de fuentes independientes}
tinicial:=0;   {tiempo incial de cada simulacin}
tfinal:=20e-3; {tiempo final de cada simulacin}

{Valores iniciales de variables de estado.
Estas se definen en el orden en que aparecen las inductancias
y Capacitancias en Cirt1}

if conmutacion=true then   {se incluyen las inductancias de linea}
begin
x0[1]:=-1.093063195;
x0[2]:=-0.046835731277;
x0[3]:=1.1398989263;
x0[4]:=0.00021276838149;
x0[5]:=-1.1347433887;
x0[6]:=1.1345306203;
end;

{valores de precisin de Simeep}
dtmax0:=20e-6;
dtmin:=dtmax0/100;
PasoDivisor:=6;
TmaxTmin:=1e3;

{indicamos a TRAXP los tiempos que debe graficar}
traxp.tinicial:=tinicial;
traxp.tfinal:=tfinal;

{En esta seccin, definimos el nmero de pantallas, y los colores,
en nmero de grillas inciales, y si tenemos alguna superpocicin
de seales en una misma ventana}

gridx:=12;	{las divisiones en X son 12}
gridy:=12;	{las diviciones en Y son 8}
subplot(2,3); {matriz de pantallas de 1 por 1}

{definiciones de visualizacin para la pantalla 0 y est asociada al canal 0}
definaY(0,-900,100);
titulo(' (Rosa): U_xy     (Amarillo): U_uv');
xlabel(' ENTRADA');
ylabel('100 V/DIV.  S  R  T');
linea0;
grid;
superponga(6,0);
superponga(7,0);
superponga(8,0);
superponga(9,0);
 definacolor(0,rojo);
 definacolor(6,negro);
 definacolor(7,azul);
 definacolor(8,14);
 definacolor(9,13);


{definiciones de visualizacin para la pantalla 1 y est asociada al canal 1}
definaY(1,-500,50);
titulo('(Amari): i_U  (Rojo): i_V  (Blan): i_X');
xlabel('30 /DIV');
ylabel('');
linea0;
grid;
superponga(10,1);
superponga(11,1);
 definacolor(1,14);
 definacolor(10,12);
 definacolor(11,15);


{definiciones de visualizacin para la pantalla 2 y est asociada al canal 2}
definaY(2,-420,70);
titulo('(Rojo): u_R (RED)  (Amari): i_R (RED)');
xlabel('30 /DIV');
ylabel('70 VOL/Div.');
linea0;
grid;
superponga(12,2);
 definacolor(2,rojo);
 definacolor(12,amarillo);


{definiciones de visualizacin para la pantalla 3 y est asociada al canal 3}
definaY(3,-240,120);
titulo('(Azul): Ud. (Marron): U1  (Amari): U2 ');
xlabel('30 /DIV');
ylabel('100 VOL/Div.      id');
linea0;
grid;
superponga(13,3);
superponga(14,3);
superponga(15,3);
 definacolor(3, 12);
 definacolor(13,1);
 definacolor(14,6);
 definacolor(15,14);

{definiciones de visualizacin para la pantalla 4 y est asociada al canal 4}
definaY(4,-120,10);
titulo('Disparos: (Verde): A  (Marron): B');
xlabel('30 /DIV');
ylabel('6.5.4.3.2.1.6.5.4.3.2.1');
linea0;
grid;
superponga(16,4);
superponga(17,4);
superponga(18,4);
superponga(19,4);
superponga(20,4);
superponga(21,4);
superponga(22,4);
superponga(23,4);
superponga(24,4);
superponga(25,4);
superponga(26,4);
 definacolor(4, 10);
 definacolor(16,10);
 definacolor(17,10);
 definacolor(18,10);
 definacolor(19,10);
 definacolor(20,10);
  definacolor(21,6);
  definacolor(22,6);
  definacolor(23,6);
  definacolor(24,6);
  definacolor(25,6);
  definacolor(26,6);

{definiciones de visualizacin para la pantalla 5 y est asociada al canal 5}
definaY(5,0,1);
titulo('Armonicos de la intensidad de fase R');
xlabel('1     5     10     15     20     25');
ylabel('');
definacolor(5,14);
END;

{---------------------------------------------------------------------}

PROCEDURE PESCA;
var   k, d, enX:integer;
		coseno:NREAL;
BEGIN
grados:=18000*t; {....Para ver en el display el tiempo y los grados.}
if grados>=360 then grados:=grados-360*Int(t/0.02);

iR_RED:=(v[20]-v[6])/(2*0.01)+(V[16]-V[1]-V[21]+V[2])/(0.01*2*sqrt(3));
mem1.tic(t,iR_RED);  {guardo los valores de la iR_red}
mem2.tic(t,v[4]);    {guardo valores de la tension de salida}

intensidad:=(v[4]-v[11])/0.1;{calculo la intensidad de salida instantanea}

			  {Calculo el tiempo de conmutacion Uk}
coseno:=cos(alpha*PI/180)-w*L_conmu*2*intensidad/ (SQRT(3)*tension);
if (coseno>-0.00001) AND (coseno<0.000001) then coseno:=0.000001;
coseno:=ABS(coseno);
U:=ArcTan(SQRT(ABS(1-SQR(coseno)))/coseno);
if alpha>90 then U:=PI-U;
U:=U*180/pi;
U:=ABS(U-alpha);

{...Pinto anotaciones sobre las graficas...}
enX:=PUNTOX+25; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Alpha}ANOTAR_EVENTO(1,30, 30+alpha-1, grados,enx+244,80,100,75,100,75,CHR(224));

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Alpha}ANOTAR_EVENTO(2,30, 30+alpha, grados,enx+244,250,250,250,250,250,CHR(224));



{indicar el tiempo de simulacin a el graficador}
traxp.t:=t;
{relacin de seales a graficar con los canales deseados}

{...........grafica 0..........}
trazo(0, (v[17]-V[10]) *0.6);       { tension de red  R }
trazo(6, (v[18]-V[10]) *0.6);       { tension de red  S }
trazo(7, (v[19]-V[10]) *0.6);       { tension de red  T }

trazo(8, (v[1]-V[2]) *0.4 -600);         {tension entre fases  u-v }
trazo(9, (v[6]-V[7]) *0.4 -600);         {tension entre fases  x-y }

{...........grafica 1..........}
trazo(1,  (v[16]-V[1])*5000 );                  { i u }
trazo(1, 0);
trazo(10, (v[21]-V[2])*5000 -200);              { i v }
trazo(10,-200);
trazo(11, (v[20]-V[6])*5000 -400);              { i x }
trazo(11,-400);

{...........grafica 2..........}
trazo(12, iR_red*100);              { i red R }
trazo(12,0);
trazo(2,  v[17]-V[10]);             { v red R }

{...........grafica 3..........}
SetLineStyle(0,0,3);
trazo(3,  (v[4]-V[11])/0.1 *1095);       { i carga   }
trazo(13, v[4]);                    { v salida  }
SetLineStyle(0,0,0);
trazo(14, v[4]-V[5]);               { v salida a}
trazo(15, v[9]);                    { v salida b}

{...........grafica 4..........}
{graficar: pulsos de disparo de cada tiristor atender los offset utilizados}
if disparo1(1)=true then d:=3 else d:=0;{veo el estado de los tiristores de A}
  trazo(4,d-10); trazo(4,-10);
if disparo1(2)=true then d:=3 else d:=0;
  trazo(16,d-20); trazo(16,-20);
if disparo1(3)=true then d:=3 else d:=0;
  trazo(17,d-30);trazo(17,-30);
if disparo1(4)=true then d:=3 else d:=0;
  trazo(18,d-40);trazo(18,-40);
if disparo1(5)=true then d:=3 else d:=0;
  trazo(19,d-50);trazo(19,-50);
if disparo1(6)=true then d:=3 else d:=0;
  trazo(20,d-60);trazo(20,-60);

if disparo2(1)=true then d:=3 else d:=0;{veo el estado de los tiristores de B}
  trazo(21,d-70); trazo(21,-70);
if disparo2(2)=true then d:=3 else d:=0;
  trazo(22,d-80); trazo(22,-80);
if disparo2(3)=true then d:=3 else d:=0;
  trazo(23,d-90);trazo(23,-90);
if disparo2(4)=true then d:=3 else d:=0;
  trazo(24,d-100);trazo(24,-100);
if disparo2(5)=true then d:=3 else d:=0;
  trazo(25,d-110);trazo(25,-110);
if disparo2(6)=true then d:=3 else d:=0;
  trazo(26,d-120);trazo(26,-120);

	{borrado de la pantalla despus de cada simulacin}
	if t >= tfinal then
	begin
			for k:= 0 to 26 do
		begin
		borrecanal(k,0);
		desligue(k);
		borde;
		if k<>5 then  {si la pantalla es la 5 (armonicos), no pintar grid}
		begin
			grid;
			linea0;
		end;
		end;
		mem1.fft;
		mem2.fft;
		{...........grafica 5..........}
		{graficar: Armonicos de corriente de la fase R de alimentacion}
		  gridx:=26;
		  PlotMem_EspectroDePotenciaBar(5,mem1,0,26,rojo,azul);
		  gridx:=12;	{las divisiones en X son 12}
		  gridy:=12;	{las diviciones en Y son 8}

		  mem1.retrocesobarrido(tfinal-tinicial);
		  valor_medio_Ud:=mem2.valormedio;
		  mem2.retrocesobarrido(tfinal-tinicial);

		t:= t-tfinal+tinicial;
	end;
END;

{---------------------------------------------------------------------}

PROCEDURE FIN;
BEGIN
HALT;
END;

{---------------------------------------------------------------------}

PROCEDURE INICIALIZACION_PARAMETROS;
BEGIN
conmutacion:=true; {indica si hay o no inductancias de conmutacion}
tension:=311;
Valor_medio_ud:=880.7;
END;
{---------------------------------------------------------------------}


{******************************PRINCIPAL*******************************}
BEGIN
INICIALIZACION_PARAMETROS;

{Indicamos a Simeep donde estn las funciones que preparamos}
FilUSR.Cirt1:=Cirt1;
FilUSR.Cirt2:=Cirt2;
FilUSR.FuncsR:=FuncsR;
FilUSR.Inicio:=Inicio;
FilUSR.Pesca:=Pesca;
FilUSR.Fin:=Fin;

{Pantalla de controles}
UCntrl.init;

mem1:= TMemoria.Create_init(20e-3/1024,1024,0,0);
mem2:= TMemoria.Create_init(20e-3/1024,1024,0,0);
{------------------PARA VISUALIZAR TEXTO EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlVerTexto,init
(    0,   '    CONVERTIDOR ',
			 '    EN CONEXION',
			 '12 PULSOS  DOS VIAS ')));

UCntrl.insert(new(PCtrlVerTexto,init
(    1,   '     ',
			 '       CARGA',
			 '  FUENTE DE I=Cte. ')));

UCntrl.insert(new(PCtrlVerTexto,init
(    8,   '     ',
			 '   ',
			 '       DISPARO')));

UCntrl.insert(new(PCtrlVerTexto,init
(    6,   '     ',
			 '',
			 '  INDUCTANCIA SERIE')));


{---------------PARA CONTROLAR VARIABLES EN LA SIMULACION--------------}

if conmutacion=true then
UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    7, ' L_CONMU',  'mH'    ,0,       1,  100,  10, 10, L_CONMU)))
else l_conmu:=0;


UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    9, '  Alpha',  ''    ,0,       0,   90,    30,  2,   alpha)));

{-----------------PARA VER VARIABLES EN LA SIMULACION------------------}

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon,  nombre1,  uni1, deci1,  var1 , nombre2,  uni2, deci2, var2}
(    2,    'Tiempo',   'Sg',   3  ,  t ,  'Grados',  '',    0 ,grados)));


UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    3, 'u',  '',      1  ,  U , 'i_d' ,  'A',   3 , intensidad)));



UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    4, 'Ud med.',  'V',     1  , valor_medio_ud , 'Ud inst.', 'V',  1,v[4])));

PASEO;
mem1.Free;
END.
