unit ueditorsala;

{$mode delphi}

interface

uses
  Classes, SysUtils,
  uConstantesSimSEE,
  usalasdejuego, ucosa, usalasdejuegoParaEditor, uInicioYFinal;

type

  { TEjecutorDeOrden }

  TEjecutorDeOrden = class
     editor: TObject; // puntero al editor sobre el que actua

     idOrden: string;
     nParMin: integer; // >= 0
     nParMax: integer; // -1 = cualquier cantidad
     constructor Create( editor: TObject; idOrden: string; nParMin, nParMax: integer );

     // chequea si el primer parámetro coincide co idOrden y si la cantidad
     // de parámetros es la adecuada y retorna TRUE si este ejecutor es apto
     // para procesar la orden y FALSE en caso contrario.
     function FirmaOk( orden: string; paramLst: TStrings ): boolean;

     function ejecutar( paramLst: TStrings ): TStrings; virtual;

     function ayuda: TStrings; virtual;

  end;



  { TRbtEditorSala_cmd }

  TRbtEditorSala_cmd = class( TRbtEditorSala )
     flg_modificada: boolean;
     EjecutoresDeOrdenes: TList;
     constructor Create;
     // ejecuta un comando
     function ejecutar( comandos: TStrings ): TStrings;
  end;

  { TEjecutor_Abrir }

  TEjecutor_Abrir = class( TEjecutorDeOrden )
     constructor Create( editor: TRbtEditorSala_cmd );
     function ejecutar( paramLst: TStrings ): TStrings; override;
     function ayuda: TStrings; override;
  end;


procedure wrln( r: TStrings; s: string );
procedure werror( r: TStrings; s: string );

implementation

procedure wrln( r: TStrings; s: string );
begin
  r.Add( s );
end;

procedure werror( r: TStrings; s: string );
begin
  r.add( 'ERROR: '+s );
end;


{ TEjecutor_Abrir }

constructor TEjecutor_Abrir.Create(editor: TRbtEditorSala_cmd);
begin
  inherited Create( editor, 'open', 1, 1 );
end;

function TEjecutor_Abrir.ejecutar(paramLst: TStrings): TStrings;
var
  res: TStrings;
  f: TArchiTexto;
  archi: string;
  pe: TRbtEditorSala_cmd;
begin
  Res:=inherited ejecutar(paramLst);
  archi:= paramLst[0];

  pe:= editor as TRbtEditorSala_cmd;

  try
      chdir(extractFilePath(archi));
      f:= TArchiTexto.CreateForRead( 0, pe.CatalogoReferencias, archi, false );
      try
        f.rd('sala', TCosa(pe.sala));
      finally
        f.Free;
      end;

      if pe.sala = nil then
        werror(res, 'No fue posible cargar la sala del archivo: '+archi );

      pe.sala.setDirCorrida(archi);
      if pe.resolverReferenciasContraSala( False) > 0 then
        werror( res, 'No se pudo resolver todas las asociaciones entre los actores' +
        ' de la sala, por lo que no se cargará. Puede ver la lista de ' +
        'referencias que no pudieron resolverse en el siguiente ' +
        'archivo: ' + getDir_Run + 'errRefs.txt' );

     pe.Sala.Prepararse_( pe.CatalogoReferencias  );
     pe.sala.publicarTodasLasVariables;

    finally
    end;
  end;


function TEjecutor_Abrir.ayuda: TStrings;
begin
  Result:=inherited ayuda;
end;

{ TRbtEditorSala }

constructor TRbtEditorSala_cmd.Create;
begin
  inherited Create( nil );
  EjecutoresDeOrdenes:= TList.Create;
end;

function TRbtEditorSala_cmd.ejecutar(comandos: TStrings): TStrings;
begin

end;

{ TEjecutorDeOrden }

constructor TEjecutorDeOrden.Create(editor: TObject; idOrden: string; nParMin,
  nParMax: integer);
begin
  inherited Create;
  self.editor:= editor;
  self.idOrden:= idOrden;
end;

function TEjecutorDeOrden.FirmaOk(orden: string; paramLst: TStrings): boolean;
begin
  result:= (idOrden = orden)
            and ( nParMin <= paramLst.count )
            and ( ( nParMax < 0 ) or ( paramLst.count <= nParMax ) ) ;
end;

function TEjecutorDeOrden.ejecutar(paramLst: TStrings): TStrings;
var
  res: TStringList;
begin
  res:= TStringList.Create;
end;

function TEjecutorDeOrden.ayuda: TStrings;
var
  res: TStringList;
  s, auxs: string;
  k: integer;
begin
  res:= TStringList.Create;
  s:= 'Sintaxis: '+idOrden;
  if nParMax = nParMin then
  begin
    if nParMax = 1 then
        s:= s +' p1'
    else if nParMax > 0 then
    begin
      for k:= 1 to nParMax do
        s:= s + ' p'+IntToStr(k);
    end;
  end
  else
  begin
    for k:= 1 to nParMin do
      s:= s +' p'+IntToStr( k );
    if nParMax < 0 then
       s:= s + '[... [p'+IntToStr( nParMin+1 )+'] ... pN]'
    else
    begin
      auxs:= '';
       for k:= nParMin + 1 to nParMax do
         auxs:= '['+auxs+' p'+IntToStr( k )+']';
      s:= s + auxs;
    end;
  end;
  res.add( s );
  result:= res;
end;


end.



constructor TRbtEditorSala.Create_Nueva;
begin

end;

procedure TRbtEditorSala.Guardar;
begin

end;

function TRbtEditorSala.ejecutar(comandos: TStrings): TStrings;
begin

end;


